@@ -24,8 +24,13 @@
 #include <mir/graphics/display.h>
 #include <mir/graphics/display_configuration.h>
 #include <mir/input/touch_visualizer.h>
+#include <mir/log.h>
+#include <mir/report_exception.h>
 
 #include <cstdio>
+#include <sstream>
+
+#include <assert.h>
 #include "screen_hardware.h"
 #include "power_state_change_reason.h"
 #include "server.h"
@@ -33,6 +38,18 @@
 namespace mi = mir::input;
 namespace mg = mir::graphics;
 
+namespace
+{
+void log_exception_in(char const* const func)
+{
+    static char const* const warning_format = "%s failed: %s";
+    std::stringstream buffer;
+
+    mir::report_exception(buffer);
+    mir::log(::mir::logging::Severity::warning, "usc::MirScreen", warning_format, func, buffer.str().c_str());
+}
+}
+
 class usc::MirScreen::LockableCallback : public mir::LockableCallback
 {
 public:
@@ -48,8 +65,7 @@ public:
 
     void unlock() override
     {
-        if (guard_lock.owns_lock())
-            guard_lock.unlock();
+        guard_lock.unlock();
     }
     
 protected:
@@ -63,7 +79,11 @@ class usc::MirScreen::PowerOffLockableCallback : public usc::MirScreen::Lockable
 
     void operator()() override
     {
-        mir_screen->power_off_alarm_notification();
+        // We need to hold the lock before calling power_off_alarm_notification_l()
+        // (and not acquire it in that function) as we override the function to
+        // test for deadlock conditions. C.f. test_deadlock_lp1491566.cpp
+        assert(guard_lock.owns_lock());
+        mir_screen->power_off_alarm_notification_l();
     }
 };
 
@@ -73,7 +93,11 @@ class usc::MirScreen::DimmerLockableCallback : public usc::MirScreen::LockableCa
 
     void operator()() override
     {
-        mir_screen->dimmer_alarm_notification();
+        // We need to hold the lock before calling dimmer_alarm_notification_l()
+        // (and not acquire it in that function) as we override the function to
+        // test for deadlock conditions. C.f. test_deadlock_lp1491566.cpp
+        assert(guard_lock.owns_lock());
+        mir_screen->dimmer_alarm_notification_l();
     }
 };
 
@@ -85,7 +109,8 @@ usc::MirScreen::MirScreen(
     std::shared_ptr<mir::time::AlarmFactory> const& alarm_factory,
     std::shared_ptr<usc::Clock> const& clock,
     Timeouts inactivity_timeouts,
-    Timeouts notification_timeouts)
+    Timeouts notification_timeouts,
+    Timeouts snap_decision_timeouts)
     : screen_hardware{screen_hardware},
       compositor{compositor},
       display{display},
@@ -98,18 +123,28 @@ usc::MirScreen::MirScreen(
               std::make_shared<DimmerLockableCallback>(this))},
       inactivity_timeouts(inactivity_timeouts),
       notification_timeouts(notification_timeouts),
+      snap_decision_timeouts(snap_decision_timeouts),
       current_power_mode{MirPowerMode::mir_power_mode_on},
       restart_timers{true},
       power_state_change_handler{[](MirPowerMode,PowerStateChangeReason){}},
-      allow_proximity_to_turn_on_screen{false}
+      allow_proximity_to_turn_on_screen{false},
+      turned_on_by_user{true}
 {
-    /*
-     * Make sure the compositor is running as certain conditions can
-     * cause Mir to tear down the compositor threads before we get
-     * to this point. See bug #1410381.
-     */
-    compositor->start();
-    reset_timers_l(PowerStateChangeReason::inactivity);
+    try
+    {
+        /*
+         * Make sure the compositor is running as certain conditions can
+         * cause Mir to tear down the compositor threads before we get
+         * to this point. See bug #1410381.
+         */
+        compositor->start();
+        reset_timers_l(PowerStateChangeReason::inactivity);
+    }
+    catch (...)
+    {
+        log_exception_in(__func__);
+        throw;
+    }
 }
 
 usc::MirScreen::~MirScreen() = default;
@@ -183,20 +218,37 @@ void usc::MirScreen::set_inactivity_timeouts(int raw_poweroff_timeout, int raw_d
 }
 
 void usc::MirScreen::set_screen_power_mode_l(MirPowerMode mode, PowerStateChangeReason reason)
+try
 {
-    if (!is_screen_change_allowed(mode, reason))
+    if (!is_screen_change_allowed_l(mode, reason))
         return;
 
     // Notifications don't turn on the screen directly, they rely on proximity events
     if (mode == MirPowerMode::mir_power_mode_on &&
-        reason == PowerStateChangeReason::notification)
+        (reason == PowerStateChangeReason::notification ||
+         reason == PowerStateChangeReason::snap_decision ||
+         reason == PowerStateChangeReason::call_done))
     {
         if (current_power_mode != MirPowerMode::mir_power_mode_on)
         {
             allow_proximity_to_turn_on_screen = true;
             screen_hardware->enable_proximity(true);
         }
-        reset_timers_ignoring_power_mode_l(reason);
+        else
+        {
+            screen_hardware->set_normal_backlight();
+        }
+
+        if (reason == PowerStateChangeReason::call_done &&
+            !turned_on_by_user)
+        {
+            reset_timers_ignoring_power_mode_l(reason, ForceResetTimers::yes);
+        }
+        else
+        {
+            reset_timers_ignoring_power_mode_l(reason, ForceResetTimers::no);
+        }
+
         return;
     }
 
@@ -214,8 +266,13 @@ void usc::MirScreen::set_screen_power_mode_l(MirPowerMode mode, PowerStateChange
         configure_display_l(mode, reason);
     }
 }
+catch (std::exception const&)
+{
+    log_exception_in(__func__);
+}
 
 void usc::MirScreen::configure_display_l(MirPowerMode mode, PowerStateChangeReason reason)
+try
 {
     if (reason != PowerStateChangeReason::proximity)
     {
@@ -230,6 +287,12 @@ void usc::MirScreen::configure_display_l(MirPowerMode mode, PowerStateChangeReas
         mode == mir_power_mode_off &&
         reason != PowerStateChangeReason::power_key;
 
+    if (mode == mir_power_mode_on)
+    {
+        turned_on_by_user = 
+            reason == PowerStateChangeReason::power_key;
+    }
+
     std::shared_ptr<mg::DisplayConfiguration> displayConfig = display->configuration();
 
     displayConfig->for_each_output(
@@ -267,6 +330,10 @@ void usc::MirScreen::configure_display_l(MirPowerMode mode, PowerStateChangeReas
     if (!power_on)
         screen_hardware->allow_suspend();
 }
+catch (std::exception const&)
+{
+    log_exception_in(__func__);
+}
 
 void usc::MirScreen::cancel_timers_l(PowerStateChangeReason reason)
 {
@@ -286,21 +353,22 @@ void usc::MirScreen::cancel_timers_l(PowerStateChangeReason reason)
 void usc::MirScreen::reset_timers_l(PowerStateChangeReason reason)
 {
     if (current_power_mode != MirPowerMode::mir_power_mode_off)
-        reset_timers_ignoring_power_mode_l(reason);
+        reset_timers_ignoring_power_mode_l(reason, ForceResetTimers::no);
 }
 
-void usc::MirScreen::reset_timers_ignoring_power_mode_l(PowerStateChangeReason reason)
+void usc::MirScreen::reset_timers_ignoring_power_mode_l(
+    PowerStateChangeReason reason, ForceResetTimers force)
 {
     if (!restart_timers)
         return;
 
-    auto const timeouts = timeouts_for(reason);
+    auto const timeouts = timeouts_for_l(reason);
     auto const now = clock->now();
 
     if (timeouts.power_off_timeout.count() > 0)
     {
         auto const new_next_power_off = now + timeouts.power_off_timeout;
-        if (new_next_power_off > next_power_off)
+        if (new_next_power_off > next_power_off || force == ForceResetTimers::yes)
         {
             power_off_alarm->reschedule_in(timeouts.power_off_timeout);
             next_power_off = new_next_power_off;
@@ -315,7 +383,7 @@ void usc::MirScreen::reset_timers_ignoring_power_mode_l(PowerStateChangeReason r
     if (timeouts.dimming_timeout.count() > 0)
     {
         auto const new_next_dimming = now + timeouts.dimming_timeout;
-        if (new_next_dimming > next_dimming)
+        if (new_next_dimming > next_dimming || force == ForceResetTimers::yes)
         {
             dimmer_alarm->reschedule_in(timeouts.dimming_timeout);
             next_dimming = new_next_dimming;
@@ -336,15 +404,25 @@ void usc::MirScreen::enable_inactivity_timers_l(bool enable)
         cancel_timers_l(PowerStateChangeReason::inactivity);
 }
 
-usc::MirScreen::Timeouts usc::MirScreen::timeouts_for(PowerStateChangeReason reason)
+usc::MirScreen::Timeouts usc::MirScreen::timeouts_for_l(PowerStateChangeReason reason)
 {
-    if (reason == PowerStateChangeReason::notification || reason == PowerStateChangeReason::proximity)
+    if (reason == PowerStateChangeReason::notification ||
+        reason == PowerStateChangeReason::proximity ||
+        reason == PowerStateChangeReason::call_done)
+    {
         return notification_timeouts;
+    }
+    else if (reason == PowerStateChangeReason::snap_decision)
+    {
+        return snap_decision_timeouts;
+    }
     else
+    {
         return inactivity_timeouts;
+    }
 }
 
-bool usc::MirScreen::is_screen_change_allowed(MirPowerMode mode, PowerStateChangeReason reason)
+bool usc::MirScreen::is_screen_change_allowed_l(MirPowerMode mode, PowerStateChangeReason reason)
 {
     if (mode == MirPowerMode::mir_power_mode_on &&
         reason == PowerStateChangeReason::proximity &&
@@ -356,15 +434,16 @@ bool usc::MirScreen::is_screen_change_allowed(MirPowerMode mode, PowerStateChang
     return true;
 }
 
-void usc::MirScreen::power_off_alarm_notification()
+void usc::MirScreen::power_off_alarm_notification_l()
 {
     configure_display_l(MirPowerMode::mir_power_mode_off, PowerStateChangeReason::inactivity);
     next_power_off = {};
 }
 
-void usc::MirScreen::dimmer_alarm_notification()
+void usc::MirScreen::dimmer_alarm_notification_l()
 {
-    screen_hardware->set_dim_backlight();
+    if (current_power_mode != MirPowerMode::mir_power_mode_off)
+        screen_hardware->set_dim_backlight();
     next_dimming = {};
 }
 
